/******************** dataDisplay.c *******************************/
/*                                                                */
/*                 VIRGO DATA DISPLAY                             */
/*                                                                */
/*   MODULE     : dataDisplay.c                                   */
/*   AUTHOR     : D. Verkindt (LAPP,Annecy)                       */
/*                verkindt@lapp.in2p3.fr                          */
/*                                                                */
/*   This application is a Data Display for the Virgo             */
/*   Gravitational Waves Interferometric Detector.                */
/*   It reads data under the frame format, either from files      */
/*   or from ethernet (using Cm encapsulation)                    */
/*   It uses the GUI Xforms and the Root library for plots,       */
/*   it allows to do various plot types, operations on channels   */
/*   and to listen to the data stream.                            */
/*                                                                */
/* Copyright (C) 2003, D. Verkindt                                */
/* For the licensing terms see the LICENSE file.                  */
/*                                                                */
/******************************************************************/

#include "dataDisplay.h"
#include "URootAccess.h"

#ifdef MEMWATCH
#    include "memwatch.h"
#endif

#include "RVersion.h"

struct DyTool* Dy;                  /* pointer to the main dataDisplay structure            */
struct DyPlot* Plot[NPLOT_MAX];     /* array of pointers to plot structures                 */
struct FrVect *UFrCurVect = NULL;   /* last vector of data read                             */
struct FrameH *inputFrame = NULL;   /* last frame read                                      */
int inputFrameSize;                 /* size in bbytes of the last frame read                */
struct soundhdr wavehdr;            /* structure to contains wave file header               */
struct timeval dytv;                /* timer to know how much time was passed in a function */
double ptime[DBGTIME_LEVELMAX+1];
char fctnamep[DBGTIME_LEVELMAX+1][256];

FD_dysliders       *dysliders;
FD_mainbrowser     *mainbrowser;
FD_mainpanel       *mainpanel;
FD_userfuncbrowser *userfuncbrowser;
FD_plotaudio       *plotaudio;
FD_plotgenepar     *plotgenepar;
FD_plot1d          *plot1d;
FD_plot2d          *plot2d;
FD_plotraw         *plotraw;
FD_plottime        *plottime;
FD_plotfft         *plotfft;
FD_plotffttime     *plotffttime;
FD_plottrfcttime   *plottrfcttime;
FD_plotcohetime    *plotcohetime;
FD_plot1dtime      *plot1dtime;
FD_plotrawtime     *plotrawtime;
FD_plottrfct       *plottrfct;
FD_plotcohe        *plotcohe;
FD_getdd           *getdd;
FL_FORM            *helppanel;
FL_FORM            *coffeepanel;
FL_FORM            *userprochelp;
FD_cmservers       *cmservers;
FD_getshm          *getshm;
FD_dyhelp          *dyhelp;
FD_dydebug         *dydebug;
FD_dyoptions       *dyoptions;
FD_dyoutput        *dyoutput;
FD_reffile         *reffile;


/*------------------------------------------------------ Main ---*/

int main (int argc, char *argv[])
{
 char dummy[LSTRING_MAX];
 int i, result;
 int myargc=1;
 char date_heure[24];
 time_t tt;

#ifdef MEMWATCH
 mwStatistics(2);
#endif

 printf("\n\n");
 printf("-----------------------------------------------------------------------------\n");
 printf("                         VIRGO Data Display (%s) \n", DYVERSION);
 printf("-----------------------------------------------------------------------------\n");
 tt = time(NULL);
 strftime(date_heure,24,"%b %d %Y  %T",localtime(&tt));
 printf("Started on:   %s  \n", date_heure); 
 printf("Compiled on:  %s  %s   Source: %s \n\n",__DATE__,__TIME__,__FILE__); 
 printf(" using:\n");
 FrLibVersion(stdout);
 printf("  ROOT %s\n", ROOT_RELEASE);
 printf("  GUI Xforms (v%dr%d)\n", FL_VERSION, FL_REVISION);
#ifndef NOCM
 printf("  Cm   (%s)\n", CMVERSION);
 printf("  DyFd (%s)\n", DYFDVERSION);
 printf("Contact: %s\n", CURRENT_MANAGER);
#endif
 printf("-----------------------------------------------------------------------------\n");
 printf("\n Usage: dataDisplay [config] -f [start GPS] -l [duration] -o [filename] -t [title] -nocm -b -q -h\n");
 printf("\n");

 /*--- Init Dy structure ---*/
 Dy = (struct DyTool *)calloc(1,sizeof(struct DyTool));
 Dy->AutoConf          = NULL;
 Dy->AutoConfFlag      = OFF;
 Dy->startTime         = DyGetGPSNow();
 Dy->duration          = -1;
 Dy->startTimeCmd      = DyGetGPSNow();
 Dy->durationCmd       = -1;
 Dy->plotsNotDisplayed = OFF;
 Dy->BgRun             = OFF;
 Dy->QuitAtStop        = OFF;
 Dy->outPicture        = NULL;
 Dy->titleCmd          = NULL;
 Dy->cmnbr             = 0;
 Dy->nocm              = OFF;

 /*--- Fill the plots and panels type names ---*/
 DyMakeTypeNames();

 /*--- Set the name of the dataDisplay ---*/
 DySetName();

 /*--- Get the configuration file, if it is provided ---*/

 if (argc>1 && strncmp(argv[1],"-",1) != 0)
    {if (is_a_file(argv[1]) == 0) {printf("\n *** %s not a file ***\n\n", argv[1]); exit(0);}
     Dy->AutoConf = my_strcpy(Dy->AutoConf,argv[1]);
     Dy->AutoConfFlag = ON;
    }

 /*--- Get the arguments passed to data Display ---*/

 for (i=1; i<argc; i++)
    {
     if (strncmp(argv[i],"-f",2) == 0 && i+1<=argc)          {Dy->startTimeCmd = atof(argv[i+1]); Dy->startTime = atof(argv[i+1]); i++;}
     else if (strncmp(argv[i],"-l",2) == 0 && i+1<=argc)     {Dy->durationCmd = atol(argv[i+1]); Dy->duration = atol(argv[i+1]); i++;}
     else if (strncmp(argv[i],"-nocm",5) == 0)                Dy->nocm = ON;
     else if (strncmp(argv[i],"-b",2) == 0)                   Dy->BgRun = ON;
     else if (strncmp(argv[i],"-q",2) == 0)                   Dy->QuitAtStop = ON;
     else if (strncmp(argv[i],"-o",2) == 0 && i+1<=argc)     {Dy->outPicture = my_strcpy(Dy->outPicture, argv[i+1]); i++;}
     else if (strncmp(argv[i],"-t",2) == 0 && i+1<=argc)     {Dy->titleCmd   = my_strcpy(Dy->titleCmd, argv[i+1]); i++;}
     else if (strncmp(argv[i],"-name",5) == 0 && i+1<=argc)  {strcpy(Dy->DyName, argv[i+1]); i++;}
     else if (strncmp(argv[i],"-noplot",7) == 0)              Dy->plotsNotDisplayed = ON;
     else if (strncmp(argv[i],"-debug",6) == 0 && i+1<=argc) {Dy->debugType = 0xffff; Dy->debug = atoi(argv[i+1]); i++;}
     else if (strncmp(argv[i],"-h",2) == 0)
        {printf(" [config]        (default=NULL)         = name of configuration file\n");
         printf("  -f [start GPS] (default=now)          = start GPS time \n");
         printf("  -l [duration]  (default=LONG_MAX)     = number of seconds of data to read \n");
         printf("  -b                                    = run in background\n");
         printf("  -q                                    = quit when dataDisplay stops\n");
         printf("  -o [filename]  (default=config)       = output picture (.jpg, .root)\n");
         printf("  -t [title]     (default=current date) = a title for the canvas of plots\n");
         printf("  -name [dyname] (default=%s)           = name of the dataDisplay (used in canvas and Cm) \n", DEFAULT_DYNAME);
         printf("  -noplot        (default=0)            = plots not displayed but dd.root file produced\n");
         printf("  -nocm          (default=0)            = no use of Cm connection \n");
         printf("  -debug [level] (default=0)            = run with debug level\n");
         printf("\n");
         exit(0);
        }
    }

 if (Dy->BgRun == ON) Dy->QuitAtStop = ON;

 if (Dy->debug > 0)
   {printf("Argument passed: \n");
    printf("  AutoConf=%s \n", Dy->AutoConf);
    printf("  startTime=%f \n", Dy->startTime);
    printf("  duration=%f \n", Dy->duration);
    printf("  BgRun=%d \n", Dy->BgRun);
    printf("  QuitAtStop=%d \n", Dy->QuitAtStop);
    printf("  outPicture=%s \n", Dy->outPicture);
    printf("  titleCmd=%s \n", Dy->titleCmd);
    printf("  name=%s \n", Dy->DyName);
    printf("  nocm=%d \n", Dy->nocm);
    printf("  noplot=%d \n", Dy->plotsNotDisplayed);
    printf("  debug=%d\n", Dy->debug);
    printf("\n");
   }

 /*--- Init the ROOT environment ---*/
 RA_InitROOT();

 /*--- Take care only of Ctrl-C ---*/

 signal (SIGINT, quitHandler);

 /*--- Init the Xform GUI panels ---*/

 DyXformIni (argv, &myargc);

 /*--- Init some internal parameters ---*/

 DyIni (argv, argc);

 /*--- Initialize the debug level in GUI with the value provided from command line ---*/

 DyXformSetDebug();

 /*--- Set the debug level of the frame library ---*/

 FrLibSetLvl(0);

 /*--- Set the function to call when a panel is closed ---*/

 DyXformSetClose();

 /*--- Connect to the Cm name server once for all ---*/

 if (Dy->nocm == OFF)
   {result = DyCmConnect();
    if (result == -1)
      {sprintf(dummy,"Not been able to connect with the Cm name %s", Dy->CmName);
       DyInfoLine(dummy);
      }
   }

 /*--- Start automatically when a configuration file is provided on command line ---*/

 if (Dy->AutoConf != NULL)
    {
     if (strstr(Dy->AutoConf,".car") != NULL)
       {printf("\n\n *** Can not read old configuration file %s ***\n\n", Dy->AutoConf);
        exit(0);
       }
     DyConfigGetFile(Dy->AutoConf);
     DyXformStart(NULL,0);
    }

/***
 while (fl_do_forms());
***/

 RA_Main();

 exit(0);
}

